/**
 * 
 */
package org.q2l.smallab;

import processing.core.*;

/**
 * @author Mike Edwards (mike@onearmedman.com)
 *
 */
public class FractionManipulativeView {

	static public int BLOCK_HEIGHT = 50;
	
	PApplet parent;
	int width, height;
	Fraction fraction;
	FractionView reducedFV, multipliedFV, resultFV;
	boolean added, common = false;
	int lcd = 1;
	int multiplier = 1;
	float r = 0,b = 0,g = 0;
	
	/**
	 * @param parent
	 * @param iwidth
	 * @param iheight
	 * @param lcd 
	 * @param fraction 
	 */
	public FractionManipulativeView(PApplet parent, int width, int height, Fraction fraction, int lcd) {
		this.parent = parent;
		this.width = width;
		this.height = height;
		this.fraction = fraction;
		this.lcd = lcd;
		this.reducedFV = new FractionView(this.parent,this.fraction.numerator(),this.fraction.denominator());
		this.multipliedFV = new FractionView(this.parent,this.multiplier,this.multiplier);
		this.resultFV = new FractionView(this.parent,
				this.fraction.numerator()*this.multiplier,
				this.fraction.denominator()*this.multiplier
				);
	}
	
	public void blockColor(float r, float g, float b)
	{
		this.r = r;
		this.g = g;
		this.b = b;
	}
	
	public int renderWidth()
	{
		return (int) (this.width/this.lcd * this.lcd);
	}
	
	public void draw()
	{
		if (this.fraction != null)
		{
			//this.beginDraw();
			
			//this.parent.background(0,0);
			
			//this.parent.stroke(this.r,this.g,this.b,255);
			this.parent.stroke(255);
			this.parent.fill(this.r,this.g,this.b,128);
			
			int div;
			
			if (this.common)
			{
				this.parent.stroke(255,255,255,255);
				div = this.width/this.lcd;
				
				for(int f = 0; f <  this.lcd * this.fraction.numerator() / this.fraction.denominator(); f++)
				{
					this.parent.line(f*div + div/2,FractionManipulativeView.BLOCK_HEIGHT * -4f,f*div + div/2,FractionManipulativeView.BLOCK_HEIGHT);
					this.parent.triangle(f*div + div/2,FractionManipulativeView.BLOCK_HEIGHT * -4f, 
							f*div + div/2 + 5,FractionManipulativeView.BLOCK_HEIGHT * -4f + 10, 
							f*div + div/2 - 5,FractionManipulativeView.BLOCK_HEIGHT * -4f + 10);
				}
			}

			//this.parent.stroke(this.r,this.g,this.b,255);				
			
			if (this.added)
			{
				this.parent.stroke(255,255,255,255);

				div = this.width/this.lcd;
				
				this.parent.fill(this.r,this.g,this.b,128);
				this.parent.fill(0,0,255,192);

				for(int f = 0; f <  this.lcd * this.fraction.numerator() / this.fraction.denominator(); f++)
				{
					this.parent.rect(f*div,FractionManipulativeView.BLOCK_HEIGHT * -4.5f,div,FractionManipulativeView.BLOCK_HEIGHT);
				}
				
				this.parent.fill(this.r,this.g,this.b,192);

				for(int f = 0; f < lcd; f++)
				{
					this.parent.rect(f*div,FractionManipulativeView.BLOCK_HEIGHT,div,FractionManipulativeView.BLOCK_HEIGHT);
				}
			}
			else
			{
				div = (int) (this.width/(this.fraction.denominator() * this.multiplier));

				this.parent.fill(this.r,this.g,this.b,128);

				for(int f = 0; f <  this.fraction.numerator() * this.multiplier; f++)
				{
					this.parent.rect(f*div,FractionManipulativeView.BLOCK_HEIGHT * -1.0f,div,FractionManipulativeView.BLOCK_HEIGHT);
				}
				
				this.parent.fill(this.r,this.g,this.b,192);

				for(int f = 0; f < this.fraction.denominator() * this.multiplier; f++)
				{
					this.parent.rect(f*div,FractionManipulativeView.BLOCK_HEIGHT,div,FractionManipulativeView.BLOCK_HEIGHT);
				}
			}
			
			this.parent.fill(255);
			
			this.parent.strokeWeight(FractionManipulativeView.BLOCK_HEIGHT);
			this.parent.line(0,FractionManipulativeView.BLOCK_HEIGHT * 0.5f,width,FractionManipulativeView.BLOCK_HEIGHT * 0.5f);

			this.parent.stroke(0);
			this.parent.fill(0);
			this.parent.strokeWeight(1);
			
			if (((PartsOfWholeDemo)this.parent).showEquations)
			{			
				this.reducedFV.n = this.fraction.numerator();
				this.reducedFV.d = this.fraction.denominator();
				
				this.reducedFV.draw(width/2f - 50.0f, 
						FractionManipulativeView.BLOCK_HEIGHT * 0.5f);
				
				this.parent.text("x",width/2 - 25,FractionManipulativeView.BLOCK_HEIGHT * 0.75f);
	
				this.multipliedFV.n = this.multiplier;
				this.multipliedFV.d = this.multiplier;
				
				this.multipliedFV.draw(width/2f + 0.0f, 
						FractionManipulativeView.BLOCK_HEIGHT * 0.5f);
							
				this.parent.text("=",width/2 + 25,FractionManipulativeView.BLOCK_HEIGHT * 0.75f);
	
				this.resultFV.n = this.fraction.numerator() * this.multiplier;
				this.resultFV.d = this.fraction.denominator() * this.multiplier;
				
				this.resultFV.draw(width/2f + 50.0f, 
						FractionManipulativeView.BLOCK_HEIGHT * 0.5f);
			
			}
			
		}
	}
	
}
