/**
 * 
 */
package org.q2l.smallab;

import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Rectangle;

import processing.core.*;
import hypermedia.net.*;
//import codeanticode.glgraphics.*;

/**
 * @author Mike Edwards (mike@onearmedman.com)
 *
 */
public class PartsOfWholeDemo extends PApplet {

	/**
	 * 
	 */
	private static final long serialVersionUID = -651155362339430280L;
	private boolean positioned = false;

	public int pwidth, pheight;
	
	public PFont numberFont;
	public Fraction firstFraction;
	public Fraction secondFraction;
	public FractionManipulativeView firstFractionView;
	public FractionManipulativeView secondFractionView;
	public FractionView firstFractionFV;
	public FractionView secondFractionFV;
	public FractionView resultFV;
	
	public boolean showEquations = false;
	
	public int lcd;
	
	public Player player1 = new Player();
	public Player player2 = new Player();
	public PlayerView playerView1 = new PlayerView(this,player1,0,255,0);
	public PlayerView playerView2 = new PlayerView(this,player2,255,128,0);
	
	private boolean onScrem;
	private UDP udpScrem;
	
	PImage backgroundImg, dripsImg, playerGreenImg, playerRedImg, targetImg, teamGreenBannerImg, teamRedBannerImg;
	
	/**
	 * 
	 */
	public PartsOfWholeDemo() {
		super();
	}

	/* (non-Javadoc)
	 * @see processing.core.PApplet#setup()
	 */
	@Override
	public void setup() {
		/*
		 * add in player active and queue areas, animated feedback, sound, etc. 
		 */
		size(1280,960);
		this.pwidth= 1024;
		this.pheight = 768;
		
		this.smooth();
		
		this.numberFont = createFont("DejaVuSansMono.ttf", 25.0f);
		this.backgroundImg = this.loadImage("background.gif");
		this.targetImg = this.loadImage("target.png");
		this.playerRedImg = this.loadImage("playerRed.png");
		this.playerGreenImg = this.loadImage("playerGreen.png");

		firstFraction = new Fraction(1,3);
		secondFraction = new Fraction(1,5);

		lcd = (int) (firstFraction.denominator() * secondFraction.denominator() / 
					Fraction.gcd(firstFraction.denominator(), secondFraction.denominator()));

		firstFractionView = new FractionManipulativeView(this,pwidth,pheight,firstFraction,lcd);
		firstFractionView.blockColor(0, 0xff, 0x98);
		// Claudio's green color #00ff98

		secondFractionView = new FractionManipulativeView(this,pwidth,pheight,secondFraction,lcd);
		secondFractionView.blockColor(0xff, 0x32, 0x99);
		// Claudio's pink color #ff3299

		firstFractionView.fraction = firstFraction;
		secondFractionView.fraction = secondFraction;
		
		this.firstFractionFV = new FractionView(this,this.firstFraction.numerator(),this.firstFraction.denominator());
		this.secondFractionFV = new FractionView(this,this.secondFraction.numerator(),this.secondFraction.denominator());
		this.resultFV = new FractionView(this,1L,1L);

		
		
		udpScrem = this.connectionInit("screm", 7502, "224.0.0.2" );

		this.noLoop();
	}
	
	
	/* (non-Javadoc)
	 * @see processing.core.PApplet#draw()
	 */
	@Override
	public void draw() {
		//autostarts the fullscreen app in the highest numbered display (e.g. projector, second-screen, etc.)
		if (!positioned)
		{
			positioned = true;
			GraphicsEnvironment environment = GraphicsEnvironment.getLocalGraphicsEnvironment();
			GraphicsDevice[] devices = environment.getScreenDevices();
			if (devices.length > 0)
			{
				Rectangle fullScreenRect = devices[devices.length - 1].getDefaultConfiguration().getBounds();
				this.frame.setBounds(fullScreenRect);
				this.frame.setLocation(fullScreenRect.x, fullScreenRect.y);
			}
		}

		this.background(this.backgroundImg);
		
		textFont(numberFont);
		if (this.lcd == this.firstFraction.denominator() * this.firstFractionView.multiplier &&
				this.lcd == this.secondFraction.denominator() * this.secondFractionView.multiplier)
		{
			this.firstFractionView.common = true;
			this.secondFractionView.common = true;
		}
		else
		{
			this.firstFractionView.common = false;
			this.secondFractionView.common = false;
			this.firstFractionView.added = false;
			this.secondFractionView.added = false;
		}

/*		String equation;
		if (this.firstFractionView.added && this.secondFractionView.added)
		{
			Fraction sum = this.firstFraction.plus(this.secondFraction);
			int multiplier = 1;
			if (sum.denominator() != lcd)
			{
				multiplier = lcd/(int)sum.denominator();
			}
			equation = this.firstFraction.numerator() * this.firstFractionView.multiplier + "/" + this.firstFraction.denominator() * this.firstFractionView.multiplier + 
				" + " + 
				this.secondFraction.numerator() * this.secondFractionView.multiplier + "/" + this.secondFraction.denominator() * this.secondFractionView.multiplier + 
				" = " + 
				sum.numerator() * multiplier + "/" + sum.denominator() * multiplier;
			resultFV.n = sum.numerator() * multiplier;
			resultFV.d = sum.denominator() * multiplier;
			if (multiplier != 1)
			{
				equation += " = ?";
			}
		} 
		else
		{
			equation = this.firstFraction.numerator() * this.firstFractionView.multiplier + "/" + this.firstFraction.denominator() * this.firstFractionView.multiplier + 
			" + " + 
			this.secondFraction.numerator() * this.secondFractionView.multiplier + "/" + this.secondFraction.denominator() * this.secondFractionView.multiplier + 
			" = ?";
		}

		this.text(equation,width/2 - this.textWidth(equation)/2,height/2+FractionManipulativeView.BLOCK_HEIGHT*1.5f);
*/		
		if (this.showEquations)
		{
			this.imageMode(CENTER);
			this.image(this.targetImg, width/2, height/2);
			this.pushMatrix();
			this.translate(0, -FractionManipulativeView.BLOCK_HEIGHT);
			this.drawEquation(width/2,height/2+FractionManipulativeView.BLOCK_HEIGHT*1.5f);			
			this.popMatrix();
		}
		this.pushMatrix();
		this.translate(0, height);
		this.rotate(PApplet.radians(180));
		//this.text(equation,width/-2 - this.textWidth(equation)/2,height/2+FractionManipulativeView.BLOCK_HEIGHT*1.5f);
		if (this.showEquations)
		{
			this.pushMatrix();
			this.translate(0, -FractionManipulativeView.BLOCK_HEIGHT);
			this.drawEquation(width/-2,height/2+FractionManipulativeView.BLOCK_HEIGHT*1.5f);			
			this.popMatrix();
		}
		this.popMatrix();

		this.pushMatrix();
		this.translate(this.firstFractionView.renderWidth() + (width-pwidth)/2, 275);
		this.rotate(PApplet.radians(180));
		firstFractionView.draw();
		this.popMatrix();

		this.pushMatrix();
		this.translate((width-pwidth)/2, height-275);
		secondFractionView.draw();
		this.popMatrix();
		
		this.playerView1.draw();
		this.playerView2.draw();

	}
	
	private void drawEquation(float x, float y)
	{
		this.fill(this.firstFractionView.r,this.firstFractionView.g, this.firstFractionView.b);
		this.stroke(this.firstFractionView.r,this.firstFractionView.g, this.firstFractionView.b);
		this.firstFractionFV.n = this.firstFraction.numerator();
		this.firstFractionFV.d = this.firstFraction.denominator();
		
		this.firstFractionFV.draw(x - 50.0f,y);
		
		this.fill(255);
		this.text("+",x - 25,y+FractionManipulativeView.BLOCK_HEIGHT * .25f);

		this.fill(this.secondFractionView.r,this.secondFractionView.g, this.secondFractionView.b);
		this.stroke(this.secondFractionView.r,this.secondFractionView.g, this.secondFractionView.b);
		this.secondFractionFV.n = this.secondFraction.numerator();
		this.secondFractionFV.d = this.secondFraction.denominator();
		
		this.secondFractionFV.draw(x,y);
		
		this.fill(255);
		this.stroke(255);
		this.text("=",x + 25,y+FractionManipulativeView.BLOCK_HEIGHT * .25f);


		if (this.firstFractionView.added && this.secondFractionView.added)
		{
			Fraction sum = this.firstFraction.plus(this.secondFraction);
			int multiplier = 1;
			
			if (sum.denominator() != lcd)
			{
				multiplier = lcd/(int)sum.denominator();
			}

			this.resultFV.n = sum.numerator() * multiplier;
			this.resultFV.d = sum.denominator() * multiplier;

			this.resultFV.draw(x + 50.0f, y);
		}
		else
		{
			this.text("?",x + 50.0f,y+FractionManipulativeView.BLOCK_HEIGHT*.25f);			
		}

	}
	
	@Override
	public void keyReleased() {
		// TODO Auto-generated method stub
		super.keyReleased();
		switch (key)
		{
		case('w'):
			this.firstFractionView.multiplier += 1;
			break;
		case('s'):
			this.firstFractionView.multiplier -= 1;
			break;
		case('a'):
			if (this.firstFractionView.common)
				this.firstFractionView.added = !this.firstFractionView.added;
			break;
		case('i'):
			this.secondFractionView.multiplier += 1;
			break;
		case('k'):
			this.secondFractionView.multiplier -= 1;
			break;
		case('j'):
			if (this.secondFractionView.common)
				this.secondFractionView.added = !this.secondFractionView.added;
			break;
		case('f'):
			this.showEquations = !this.showEquations;
			break;
		case(' '):
			randomGame();
		}
		this.firstFractionView.multiplier = PApplet.constrain(this.firstFractionView.multiplier, 1, 20);
		this.secondFractionView.multiplier = PApplet.constrain(this.secondFractionView.multiplier, 1, 20);
		this.redraw();
	}
	
	public void randomGame()
	{
		int d1,n1,d2,n2;
		d1 = (int)this.random(2.0f, 9.0f);
		n1 = (int)this.random(1.0f, d1/2.0f);
		d2 = (int)this.random(2.0f, 9.0f);
		n2 = (int)this.random(1.0f, d2/2.0f);
		
		this.firstFraction = new Fraction(n1,d1);
		this.secondFraction = new Fraction(n2,d2);

		lcd = (int) (firstFraction.denominator() * secondFraction.denominator() / 
				Fraction.gcd(firstFraction.denominator(), secondFraction.denominator()));

		this.firstFractionView.fraction = this.firstFraction;
		this.secondFractionView.fraction = this.secondFraction;
		
		this.firstFractionView.lcd = this.secondFractionView.lcd = this.lcd;
		
		this.firstFractionView.multiplier = this.secondFractionView.multiplier = 1;
	}
	
	public UDP connectionInit(String name, int port,String address)
	{
		UDP udp;
		
		udp = new UDP(this,port,address);
		udp.setReceiveHandler("receive_" + name);
		
		udp.setBuffer(2048);
		// wait constantly for incoming data
		udp.listen( true );
		// ... well, just verifies if it's really a multicast socket and blablabla
		println( "init as multicast socket ... "+udp.isMulticast() );
		println( "joins a multicast group  ... "+udp.isJoined() );
		
		return udp;
	}

	public void receive_screm( byte[] data ) {
		if(!onScrem){
			onScrem=true;
			println("SCREM is ON!!");
		}
		String k="";
		for(int i=0;i<data.length;i++){
			k+=(char)data[i];
		}
		String[] g= k.split(" ");
		this.println(k);
		if(g[0].equals("JavaREM"))
		{
			if (g[3].equals("ogball"))
			{
				if (g[4].equals("gx"))
				{
					player1.x = Float.parseFloat(g[5]) * this.width;					
				}
				if (g[4].equals("gy"))
				{
					player1.y = this.height - Float.parseFloat(g[5]) * this.height;					
				}
				if (g[4].equals("gz"))
				{
					player1.z = Float.parseFloat(g[5]) * this.height;					
				}

				if (g[4].equals("ox"))
				{
					player2.x = Float.parseFloat(g[5]) * this.width;				
				}
				if (g[4].equals("oy"))
				{
					player2.y = this.height - Float.parseFloat(g[5]) * this.height;				
				}
				if (g[4].equals("oz"))
				{
					player2.z = Float.parseFloat(g[5]) * this.height;					
				}

				if (g[4].equals("pause") && g[5].equals("1.0"))
				{
					println("pause hit");
					this.showEquations = !this.showEquations;
				}
				if (g[4].equals("reset") && g[5].equals("1.0"))
				{
					println("reset hit");	
					this.randomGame();
				}
				if (g[4].equals("resume") && g[5].equals("1.0"))
				{
					println("resume hit");					
				}
				this.redraw();
			}
		}
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		//autostarts the fullscreen app in the highest numbered display (e.g. projector, second-screen, etc.)
		GraphicsEnvironment environment = GraphicsEnvironment.getLocalGraphicsEnvironment();
		GraphicsDevice[] devices = environment.getScreenDevices();

		String displayOption = "--display=" + devices.length;
		
		PApplet.main(new String[] {"--present",displayOption,"--bgcolor=#000000","org.q2l.smallab.PartsOfWholeDemo"});
	}

}
